/* docklet.c
 * Contributed by Line72 <line72@line72.homelinux.com>
 */

#include "docklet.h"

gboolean docklet_create(GtkWidget *win) {
  GtkWidget *box;

  main_window = win;

  status = stopped;

  docklet = egg_tray_icon_new("GQueue");
  box = gtk_event_box_new();
  image = gtk_image_new();

  g_signal_connect(G_OBJECT(docklet), "embedded", G_CALLBACK(docklet_embedded), NULL);
  g_signal_connect(G_OBJECT(docklet), "destroy", G_CALLBACK(docklet_destroyed), NULL);
  g_signal_connect(G_OBJECT(box), "button-press-event", G_CALLBACK(docklet_clicked), NULL);

  gtk_container_add(GTK_CONTAINER(box), image);
  gtk_container_add(GTK_CONTAINER(docklet), box);
  gtk_widget_show_all(GTK_WIDGET(docklet));

  g_object_ref(G_OBJECT(docklet));

  docklet_update_status(status);
  docklet_update_icon();

  return FALSE;
}

void docklet_update_status(enum docklet_status new_status) {
  if (new_status != status) {
    if (new_status == printing) 
      status = printing;
    else 
      status = stopped;

    docklet_update_icon();
  }
  
  return FALSE;
}

static void docklet_update_icon() {
  const gchar *icon_name = NULL;
  
  switch(status) {
  case stopped:
    icon_name = "gqueue-docklet";
    break;
  case printing:
    icon_name = "gqueue-docklet-printing";
    break;
  }

  gtk_image_set_from_stock(GTK_IMAGE(image), icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
}

void docklet_clicked(GtkWidget *button, GdkEventButton *event, void *data) {
	
	GtkWidget *menu;
	
  if (event->type != GDK_BUTTON_PRESS)
    return;

  if (event->button == 3)
	{
		menu = create_context_menu_icon();
	    gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
						  event->button, event->time);
	}
  else docklet_toggle();
}

void docklet_toggle() {
  if (gdk_window_get_state(GTK_WIDGET(main_window)->window) & GDK_WINDOW_STATE_ICONIFIED)
    docklet_unhide();
  else
    docklet_hide();
}

static void docklet_hide() {
  gtk_widget_hide(main_window);
  gtk_window_iconify(GTK_WINDOW(main_window));
}

static void docklet_unhide() {
  gtk_widget_show(main_window);
  gtk_window_present(GTK_WINDOW(main_window));
}

static void docklet_register_icon(const char *name, char *fn) {
  gchar *filename;

  filename = g_build_filename(DATADIR, "pixmaps", "gqueue", fn, NULL);
  gtk_icon_factory_add(icon_factory, name,
		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_file(filename, NULL)));

  g_free(filename);
}

static void docklet_register_icon_filename(const char *name, char *fn) {
  gchar *filename;

  filename = g_build_filename(fn, NULL);
  gtk_icon_factory_add(icon_factory, name,
		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_file(filename, NULL)));

  g_free(filename);
}

void docklet_register_icon_factory() {
  icon_factory = gtk_icon_factory_new();

  docklet_register_icon("gqueue-docklet", "default/gqueueicon.png");
  docklet_register_icon("gqueue-docklet-printing", "default/gqueue-printing.png");

  gtk_icon_factory_add_default(icon_factory);
}

void docklet_register_icon_factory_skin(char *name)
{
  gchar *filename, *filename_printing;

  icon_factory = gtk_icon_factory_new();

  filename = g_build_filename(name,"gqueue.png", NULL);
  filename_printing = g_build_filename(name,"gqueue-printing.png", NULL);	
  docklet_register_icon_filename("gqueue-docklet", filename);
  docklet_register_icon_filename("gqueue-docklet-printing", filename_printing);

  gtk_icon_factory_add_default(icon_factory);
}

static void docklet_unregister_icon_factory() {
  gtk_icon_factory_remove_default(icon_factory);
}

static void docklet_remove_callbacks() {
  while (g_source_remove_by_user_data(&docklet)) {

  }
}

static void docklet_embedded(GtkWidget *widget, void *data) {

}

static void docklet_destroyed(GtkWidget *widget, void *data) {
  docklet_remove_callbacks();

  g_object_unref(G_OBJECT(docklet));
  docklet = NULL;

  g_idle_add(docklet_create, &docklet);
}

void docklet_delete() {
  docklet_remove_callbacks();

  g_signal_handlers_disconnect_by_func(G_OBJECT(docklet), G_CALLBACK(docklet_destroyed), NULL);
  gtk_widget_destroy(GTK_WIDGET(docklet));

  g_object_unref(G_OBJECT(docklet));
  docklet = NULL;

  docklet_unregister_icon_factory();
}
